from .linkedin import ChallengeException, LinkedInMessaging

__title__ = "linkedin_messaging"
__version__ = "0.1.0"
__description__ = "An unofficial API for interacting with LinkedIn Messaging"

__license__ = "MIT"

__author__ = "Sumner Evans"
__email__ = "inquiries@sumnerevans.com"

__all__ = ("ChallengeException", "LinkedInMessaging")
